from transformers import AutoModelForCausalLM, AutoTokenizer
from peft import PeftModel
import torch

# Check GPU usage
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
print(f"Using device: {device}")

# Paths
base_model_path = r"C:\Users\lucas\aait_store\Models\NLP\Chocolatine-3B-Instruct"
lora_adapter_path = r"C:\Users\lucas\AppData\Local\Programs\Orange_dev\Lib\results\checkpoint-15525"
merged_model_output_path = r"C:\Users\lucas\merged_model"

# Clean the output directory (if needed)
import shutil, os
if os.path.exists(merged_model_output_path):
    shutil.rmtree(merged_model_output_path)
os.makedirs(merged_model_output_path, exist_ok=True)

# Load the base model
base_model = AutoModelForCausalLM.from_pretrained(base_model_path, torch_dtype=torch.float16)

# Load the LoRA adapter
lora_model = PeftModel.from_pretrained(base_model, lora_adapter_path)

# Merge LoRA weights into the base model
lora_model.merge_and_unload()

# Save only the merged base model
base_model.save_pretrained(merged_model_output_path)

# Save the tokenizer
tokenizer = AutoTokenizer.from_pretrained(base_model_path)
tokenizer.save_pretrained(merged_model_output_path)

print(f"Merged model saved to {merged_model_output_path}")








# # Test function to evaluate before and after fine-tuning
# def test_model(model, tokenizer, test_queries):
#     model.eval()  # Set the model to evaluation mode
#     for query in test_queries:
#         prompt = f"### User: {query}\n\n### Assistant:"
#
#         # Move input_ids to the appropriate device
#         input_ids = tokenizer(prompt, return_tensors="pt").input_ids.to(device)
#
#         # Generate output on the GPU
#         output = model.generate(input_ids, max_length=400, temperature=0, top_p=0)
#
#         # Decode the response
#         response = tokenizer.decode(output[0], skip_special_tokens=True)
#         response = response.replace(prompt, "")
#
#         print(f"Query: {query}")
#         print(f"Response: {response}\n")
#         print("#############")


# # Move the model to GPU
# base_model.to(device)
#
# # Test model before fine-tuning
# print("Testing model before fine-tuning:")
# test_queries = [
#     "Pour l'hélicoptère FAMA K209M, quelles sont les vitesses recommandées pour : le décollage, l'autorotation, et la vitesse d'approche ?",
#     "Pour l'hélicoptère FAMA K209M, quelles sont les étapes de l'autorotation avec remise de puissance ?",
#     "Pour l'hélicoptère FAMA K209M, quelles sont les étapes pour le remplacement du filtre à huile de turbine ?",
#     "Pour l'hélicoptère FAMA K209M, quelles sont les mesures à prendre en cas d'incendie ?",
#     "Quelle est la référence de l'huile moteur de l'hélicoptère FAMA K209M ?"
# ]
# test_model(base_model, tokenizer, test_queries)




import copy
from threading import Thread

from Orange.data import Domain, StringVariable, Table
from transformers import TextIteratorStreamer


def generate_translation(table, model, tokenizer, progress_callback=None, argself=None):
    """
    generate a translation
    input : datas in content column in a table
    output : input table with translation in "Translation" column
    """
    # Copy of input data
    data = copy.deepcopy(table)
    attr_dom = list(data.domain.attributes)
    metas_dom = list(data.domain.metas)
    class_dom = list(data.domain.class_vars)

    # Generate translation on column named "content"
    try:
        rows = []
        for i, row in enumerate(data):
            features = list(data[i])
            metas = list(data.metas[i])
            translation = translate(str(row["content"]), model=model, tokenizer=tokenizer, stream=True,
                                    argself=argself)
            metas += [translation]
            rows.append(features + metas)
            if progress_callback is not None:
                progress_value = float(100 * (i + 1) / len(data))
                progress_callback(progress_value)
            if argself is not None:
                if argself.stop:
                    break
    except ValueError as e:
        print("An error occurred when trying to generate a translation:", e)
        return

    # Generate new Domain to add to data
    var_name = "Translation"
    count = 1
    while var_name in data.domain:
        var_name = f"Translation ({count})"
        count += 1
    answer_dom = [StringVariable(var_name)]

    # Create and return table
    domain = Domain(attributes=attr_dom, metas=metas_dom + answer_dom, class_vars=class_dom)
    out_data = Table.from_list(domain=domain, rows=rows)
    return out_data


def translate(text, model, tokenizer, stream=False, argself=None):
    tokenized_text = tokenizer([text], return_tensors="pt")
    if not stream:
        tokenized_translated = model.generate(**tokenized_text)
        translated_text = tokenizer.decode(tokenized_translated[0], skip_special_tokens=True)
    else:  # TODO: décalage bizarre au lancement d'un workflow
        streamer = TextIteratorStreamer(tokenizer, skip_special_tokens=True)
        generation_kwargs = dict(tokenized_text, streamer=streamer, max_new_tokens=512, num_beams=1)
        thread = Thread(target=model.generate, kwargs=generation_kwargs)
        thread.start()
        translated_text = ""
        for token in streamer:
            translated_text += token
            if argself is not None:
                if argself.stop:
                    break
    return translated_text



import os
import sys

from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output
from Orange.data import Table, Domain, ContinuousVariable
from thefuzz import fuzz
from AnyQt.QtWidgets import QApplication

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
    from Orange.widgets.orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
else:
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file

@apply_modification_from_python_file(filepath_original_widget=__file__)
class OWKeywords(widget.OWWidget):
    name = "Keywords Detection"
    description = "Give the amount of keywords from in_object in in_data"
    icon = "icons/keyword.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/keyword.png"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owkeyword.ui")
    want_control_area = False
    priority = 1050

    class Inputs:
        data = Input("Content", Table)
        keywords = Input("Keywords", Table)

    class Outputs:
        data = Output("Keywords per Content", Table)

    def __init__(self):
        super().__init__()
        self.data = None
        self.keywords = None
        self.setFixedWidth(470)
        self.setFixedHeight(300)
        uic.loadUi(self.gui, self)
        self.autorun = True

    @Inputs.data
    def set_data(self, in_data):
        self.data = in_data
        if self.autorun and self.keywords:
            self.process()

    @Inputs.keywords
    def set_keywords(self, in_keywords):
        self.keywords = in_keywords
        if self.autorun and self.data:
            self.process()

    def fuzzy_match_score_with_keywords(self, text, keywords, threshold=80):
        """
        Returns fuzzy global score and matched keywords.
        """
        words = text.split(" ")
        total_score = 0
        matched_keywords = []

        for keyword in keywords:
            best_score = max(fuzz.ratio(word.lower(), keyword.lower()) for word in words)
            if best_score >= threshold:
                matched_keywords.append(keyword)
                total_score += best_score

        if not matched_keywords:
            return 0.0, []

        avg_score = total_score / len(keywords)
        return avg_score, matched_keywords

    def extract_matched_keywords(self, text, keywords, threshold=80):
        """
        Extracts matched keywords from a text using fuzzy matching.

        Args:
            text (str): The input string to analyze.
            keywords (list): A list of keywords to match against.
            threshold (int): Minimum fuzzy ratio to consider a match.

        Returns:
            list: Keywords that match the text above the threshold.
        """
        words = text.split(" ")
        matched_keywords = []

        for keyword in keywords:
            best_score = max(fuzz.ratio(word.lower(), keyword.lower()) for word in words)
            if best_score >= threshold:
                matched_keywords.append(keyword)

        return matched_keywords

    def process(self):
        if self.data is None or self.keywords is None:
            print("[INFO] No input data or keywords provided.")
            return

        if "Content" not in self.data.domain:
            self.error("Missing 'Content' column in input data")
            print("[ERROR] 'Content' column not found in input data.")
            return

        self.error("")  # Clear previous errors

        # Extraction des mots-clés
        try:
            if "Keywords" not in self.keywords.domain:
                self.error("Missing 'Keywords' column in keywords table")
                print("[ERROR] 'Keywords' column not found in keyword input.")
                return

            keyword_list = [str(row["Keywords"]) for row in self.keywords if str(row["Keywords"]).strip() != ""]
            print(f"[INFO] Extracted {len(keyword_list)} keywords: {keyword_list}")
        except Exception as e:
            self.error("Error while extracting keywords")
            print(f"[EXCEPTION] Failed to extract keywords: {e}")
            return

        # Création du nouveau domaine avec uniquement la colonne "Keywords"
        new_metas_vars = list(self.data.domain.metas)

        if not any(var.name == "Keywords" for var in new_metas_vars):
            new_metas_vars.append(ContinuousVariable("Keywords"))

        new_domain = Domain(
            self.data.domain.attributes,
            self.data.domain.class_vars,
            new_metas_vars
        )

        # Fonction de score
        def fuzzy_match_score(text, keywords, threshold=80):
            words = text.split(" ")
            total_score = 0
            matched = 0

            for keyword in keywords:
                best_score = max(fuzz.ratio(word.lower(), keyword.lower()) for word in words)
                if best_score >= threshold:
                    total_score += best_score
                    matched += 1

            if matched == 0:
                return 0.0
            return total_score / len(keywords)

        # Construction des nouvelles métadonnées
        new_metas = []
        for i, row in enumerate(self.data):
            text = str(row["Content"])
            score = fuzzy_match_score(text, keyword_list)
            print(f"[DEBUG] Row {i}: Score = {score:.2f}")
            new_metas.append(list(row.metas) + [score])

        out_data = Table(new_domain, self.data.X, self.data.Y, new_metas)
        print("[INFO] Process complete. Sending output data.")
        self.Outputs.data.send(out_data)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    ow = OWKeywords()
    ow.show()
    app.exec_()



import ctypes
import os
import sys
import Orange.data
from Orange.widgets.widget import Input, Output, OWWidget


class EndLoopWidget(OWWidget):
    name = "End Loop"
    description = "Widget to end a loop based on a predefined condition."
    icon = "icons/endloop.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/endloop.png"

    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owendloop.ui")
    want_control_area = False
    priority = 1011

    class Inputs:
        in_data = Input("Data In", Orange.data.Table)
        in_pointer = Input("End of the Loop Do-While", str, auto_summary=False)

    class Outputs:
        out_data = Output("Data Out", Orange.data.Table)

    def __init__(self):
        super().__init__()
        self.data = None
        self.in_pointer = None

    @Inputs.in_data
    def set_data(self, data):
        self.error("")
        if data is None:
            self.data = None
            return
        if self.in_pointer is None:
            return

        self.data = data


        # Check if the number of lines has changed
        if self.get_nb_line() != self.get_nb_line_from_start():
            self.error("Error! You can't change the number of lines")
            return

        #check domain
        column_names_input_temp, column_types_input=self.get_column_name_and_type_from_start()
        column_names_output_temp, column_types_ouput = self.get_column_name_and_type()

        column_names_input=[]
        column_names_output = []
        for idx,element in enumerate(column_names_input_temp):
            column_names_input.append(element+ " ("+column_types_input[idx]+")")

        for idx, element in enumerate(column_names_output_temp):
            column_names_output.append(element + " (" + column_types_ouput[idx] + ")")

        if column_names_input != column_names_output:
            # Find elements that are only in one of the lists
            diff1 = [x for x in column_names_input if x not in column_names_output]
            diff2 = [x for x in column_names_output if x not in column_names_input]

            # Detect swapped elements
            swapped = [(x, y) for x, y in zip(column_names_input, column_names_output) if
                       x != y and x in column_names_output and y in column_names_input]

            # Function to format a list as a sentence
            def list_to_sentence(lst):
                if not lst:
                    return "None"
                if len(lst) == 1:
                    return lst[0]
                return ", ".join(lst[:-1]) + " and " + lst[-1]

            # Build the output message
            sentence = ""
            if diff1:
                sentence += f"The elements that are only in the beginning of the loop table are: {list_to_sentence(diff1)}.\n"
            if diff2:
                sentence += f"The elements that are only in the end of the loop table are: {list_to_sentence(diff2)}.\n"

            if swapped:
                swapped_sentence = ", ".join([f"{x} <-> {y}" for x, y in swapped])
                sentence += f"Detected swapped elements: {swapped_sentence}."
            self.error(sentence)
            return

        # input of the loop== output of the loop
        old_table=ctypes.cast(int(self.in_pointer), ctypes.py_object).value.get_in_data()
        idem=True
        for idx,element in enumerate(old_table):
            if element!=self.data[idx]:
                idem=False
                break

        # check if loop doesn't change anithing
        if idem:
            self.error("Error! your loop didn't change the data table")
            return




        if any(x > 0 for x in self.data.get_column("iter")):
            ctypes.cast(int(self.in_pointer), ctypes.py_object).value.set_data(self.data)
            return
        self.Outputs.out_data.send(self.data)




    @Inputs.in_pointer
    def set_pointer(self, pointer):
        self.in_pointer = int(pointer) if pointer else None

    def get_column_name_and_type_from_start(self):
        if self.in_pointer is not None:
            result = ctypes.cast(int(self.in_pointer), ctypes.py_object).value.get_column_name_and_type()
        return result

    def get_nb_line_from_start(self):
        result = 0
        if self.in_pointer is not None:
            result = ctypes.cast(int(self.in_pointer), ctypes.py_object).value.get_nb_line()
        return result

    def get_nb_line(self):
        # Return the number of lines to compare with another widget
        if self.data is None:
            return 0
        return len(self.data)

    def get_column_name_and_type(self):
        # Return the name and type of 'data_in' to verify if they are the same
        if self.data is None:
            return [[], []]
        column_names = []
        column_types = []
        for element in self.data.domain:
            column_names.append(str(element))
            column_types.append(str(type(element)))
        return column_names, column_types





if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication
    app = QApplication(sys.argv)
    obj = EndLoopWidget()
    obj.show()
    app.exec_()






# tested only witexpected_format_type StringVariable ContinuousVariable
def check_data(dataset, expected_column_name, expected_format_type):
    """
    usage :
    if not check_data_in.check_data(dataset, "col_name", "StringVariable"):
        print("error")
        return
    -----
    return True if dataset ahas a column with title == expected_column_name (string)
                and if formats == expected "StringVariable" , "ContinuousVariable"
    else return False
    """
    if expected_column_name not in dataset.domain:
        return False
    variable_domain = dataset.domain[expected_column_name]
    if str(type(variable_domain))==expected_format_type:
        return True
    return False

def check_all_data(argself,dataset,list_expect_column_name,list_expect_format_type):
    """
    usage :
    if not check_data_in.check_all_data(self, dataset, ["col1_name", "col2_name"],
                                   ["StringVariable", "ContinuousVariable"]):
        return
   call check_data for all
   True if check is ok else False (in this case print error on widget)
    """
    if len(list_expect_column_name)!=len(list_expect_format_type):
        print("error dev not possible")
        return False
    list_error=""
    for idx in range(len(list_expect_column_name)):
        if check_data(dataset,list_expect_column_name[idx],list_expect_format_type[idx]):
            continue
        if list_error!="":
            list_error=list_error+"\n"
        list_error=list_error+"error with name/type column -> "+str(list_expect_column_name[idx])
    argself.error(list_error)
    if list_error=="":
        return True
    return False




import subprocess


workflow_key = "Acuity_RAG"


# Step 1: Start the workflow
start_command = [
    "curl",
    "-X", "GET",
    f"http://localhost:8000/start-workflow/{workflow_key}",
    "-H", "accept: application/json"
]

start_result = subprocess.run(start_command, capture_output=True, text=True)
print("Start Workflow - Status Code:", start_result.returncode)
print("Start Workflow - Output:", start_result.stdout)


import os
import sys

import Orange.data
from AnyQt.QtWidgets import QApplication
from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output
from transformers import MarianMTModel

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils import thread_management
    from Orange.widgets.orangecontrib.AAIT.llm import translations
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
else:
    from orangecontrib.AAIT.utils import thread_management
    from orangecontrib.AAIT.llm import translations
    from orangecontrib.AAIT.utils.import_uic import uic


class OWTranslation(widget.OWWidget):
    name = "Translation"
    description = "Generate a translation on the column 'content' of a Table"
    icon = "icons/owtranslation.svg"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/owtranslation.svg"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owtranslation.ui")
    want_control_area = False
    priority= 1100
    class Inputs:
        data = Input("Data", Orange.data.Table)
        model = Input("Model", (MarianMTModel, object), auto_summary=False)

    class Outputs:
        data = Output("Data", Orange.data.Table)

    @Inputs.data
    def set_data(self, in_data):
        self.data = in_data
        self.run()

    @Inputs.model
    def set_model(self, in_model):
        if in_model is not None:
            self.model = in_model[0]
            self.tokenizer = in_model[1]
        else:
            self.model = None
            self.tokenizer = None
        self.run()

    def __init__(self):
        super().__init__()
        # Path management
        self.current_ows = ""

        # Qt Management
        self.setFixedWidth(470)
        self.setFixedHeight(300)
        uic.loadUi(self.gui, self)

        # Data Management
        self.data = None
        self.model = None
        self.tokenizer = None
        self.thread = None
        self.result = None

    def run(self):
        # if thread is running quit
        if self.thread is not None:
            self.thread.safe_quit()

        if self.data is None:
            return

        if self.model is None:
            return

        if self.tokenizer is None:
            return

        # Verification of in_data
        self.error("")
        try:
            self.data.domain["content"]
        except KeyError:
            self.error('You need a "content" column in input data')
            return

        if type(self.data.domain["content"]).__name__ != 'StringVariable':
            self.error('"content" column needs to be a Text')
            return

        # Start progress bar
        self.progressBarInit()

        # Connect and start thread : main function, progress, result and finish
        # --> progress is used in the main function to track progress (with a callback)
        # --> result is used to collect the result from main function
        # --> finish is just an empty signal to indicate that the thread is finished
        self.thread = thread_management.Thread(translations.generate_translation,
                                               self.data, self.model, self.tokenizer)
        self.thread.progress.connect(self.handle_progress)
        self.thread.result.connect(self.handle_result)
        self.thread.finish.connect(self.handle_finish)
        self.thread.start()

    def handle_progress(self, value: float) -> None:
        self.progressBarSet(value)

    def handle_result(self, result):
        try:
            self.result = result
            self.Outputs.data.send(self.result)
        except Exception as e:
            print("An error occurred when sending out_data:", e)
            self.Outputs.data.send(None)
            return

    def handle_finish(self):
        print("Generation finished")
        self.progressBarFinished()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = OWTranslation()
    my_widget.show()
    app.exec_()



import os

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\","/"):
    from Orange.widgets.orangecontrib.AAIT.utils import MetManagement
    from Orange.widgets.orangecontrib.AAIT.utils.tools import (
        change_owcorpus, concat_splitted_pypi)
else:
    from orangecontrib.AAIT.utils import MetManagement
    from orangecontrib.AAIT.utils.tools import (change_owcorpus,
                                                concat_splitted_pypi)
concat_splitted_pypi.unzip_dependancy_if_needed(concat_splitted_pypi.get_path_of_OrangeDir()+"/../aait_store",concat_splitted_pypi.get_path_of_OrangeDir()+"/../aait_store/Parameters/requirements.json",concat_splitted_pypi.get_site_package_path()+"aait_store_cut-part_001/input/aait_store.zip.001",16)
concat_splitted_pypi.unzip_dependancy_if_needed(MetManagement.get_local_store_path()+"Models/NLP/all-mpnet-base-v2",MetManagement.get_local_store_path()+"Models/NLP/all-mpnet-base-v2/model.safetensors",concat_splitted_pypi.get_site_package_path()+"all-mpnet-base-v2-pypi-part_001/input/all-mpnet-base-v2.zip.001",5)
concat_splitted_pypi.unzip_dependancy_if_needed(concat_splitted_pypi.get_path_of_OrangeDir()+"/Lib/site-packages/forall/gpt4all",concat_splitted_pypi.get_path_of_OrangeDir()+"/Lib/site-packages/forall/gpt4all/bin/chat.exe",concat_splitted_pypi.get_site_package_path()+"gpt4all-pypi-part_001/input/gpt4all.zip.001",0)
change_owcorpus.replace_owcorpus_file()


import os
import sys
from Orange.data import Table, Domain, StringVariable
from AnyQt.QtWidgets import QApplication
from AnyQt.QtCore import QEventLoop

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\","/"):
     from Orange.widgets.orangecontrib.AAIT.widgets.OWChunking import OWChunker
     from Orange.widgets.orangecontrib.AAIT.audit_widget.SignalReceiver import SignalReceiver
else:
     from orangecontrib.AAIT.audit_widget.SignalReceiver import SignalReceiver
     from orangecontrib.AAIT.widgets.OWChunking import OWChunker


def check_create_chunker(chunker, receiver):
     text_var = StringVariable("content")
     domain = Domain([], metas=[text_var])
     table = Table.from_list(domain, [["In cognitive psychology, chunking is a process by which small individual pieces of a set of information are bound together to create a meaningful whole later on in memory.[1] The chunks, by which the information is grouped, are meant to improve short-term retention of the material, thus bypassing the limited capacity of working memory and allowing the working memory to be more efficient.[2][3][4] A chunk is a collection of basic units that are strongly associated with one another, and have been grouped together and stored in a person's memory. These chunks can be retrieved easily due to their coherent grouping.[5] It is believed that individuals create higher-order cognitive representations of the items within the chunk. The items are more easily remembered as a group than as the individual items themselves. These chunks can be highly subjective because they rely on an individual's perceptions and past experiences, which are linked to the information set. The size of the chunks generally ranges from two to six items but often differs based on language and culture.[6] According to Johnson (1970), there are four main concepts associated with the memory process of chunking: chunk, memory code, decode and recode.[7] The chunk, as mentioned prior, is a sequence of to-be-remembered information that can be composed of adjacent terms. These items or information sets are to be stored in the same memory code. The process of recoding is where one learns the code for a chunk, and decoding is when the code is translated into the information that it represents. The phenomenon of chunking as a memory mechanism is easily observed in the way individuals group numbers, and information, in day-to-day life. For example, when recalling a number such as 12101946, if numbers are grouped as 12, 10, and 1946, a mnemonic is created for this number as a month, day, and year. It would be stored as December 10, 1946, instead of a string of numbers. Similarly, another illustration of the limited capacity of working memory as suggested by George Miller can be seen from the following example: While recalling a mobile phone number such as 9849523450, we might break this into 98 495 234 50. Thus, instead of remembering 10 separate digits that are beyond the putative seven plus-or-minus two memory span, we are remembering four groups of numbers.[8] An entire chunk can also be remembered simply by storing the beginnings of a chunk in the working memory, resulting in the long-term memory recovering the remainder of the chunk.[4] Modality effect A modality effect is present in chunking. That is, the mechanism used to convey the list of items to the individual affects how much chunking occurs. Experimentally, it has been found that auditory presentation results in a larger amount of grouping in the responses of individuals than visual presentation does. Previous literature, such as George Miller's The Magical Number Seven, Plus or Minus Two: Some Limits on our Capacity for Processing Information (1956) has shown that the probability of recall of information is greater when the chunking strategy is used.[8] As stated above, the grouping of the responses occurs as individuals place them into categories according to their inter-relatedness based on semantic and perceptual properties. Lindley (1966) showed that since the groups produced have meaning to the participant, this strategy makes it easier for an individual to recall and maintain information in memory during studies and testing.[9] Therefore, when chunking is used as a strategy, one can expect a higher proportion of correct recalls.Memory training systems, mnemonicVarious kinds of memory training systems and mnemonics include training and drills in specially-designed recoding or chunking schemes.[10] Such systems existed before Miller's paper, but there was no convenient term to describe the general strategy and no substantive and reliable research. The term chunking is now often used in reference to these systems. As an illustration, patients with Alzheimer's disease typically experience working memory deficits; chunking is an effective method to improve patients' verbal working memory performance.[11] Patients with schizophrenia also experience working memory deficits which influence executive function; memory training procedures positively influence cognitive and rehabilitative outcomes.[12] Chunking has been proven to decrease the load on the working memory in many ways. As well as remembering chunked information easier, a person can also recall other non-chunked memories easier due to the benefits chunking has on the working memory.[4] For instance, in one study, participants with more specialized knowledge could reconstruct sequences of chess moves because they had larger chunks of procedural knowledge, which means that the level of expertise and the sorting order of the information retrieved is essential in the influence of procedural knowledge chunks retained in short-term memory.[13] Chunking has been shown to have an influence in linguistics, such as boundary perception.[14]Efficient Chunk sizesAccording to the research conducted by Dir lam (1972), a mathematical analysis was conducted to see what the efficient chunk size is. We are familiar with the size range that chunking holds, but Dir lam (1972) wanted to discover the most efficient chunk size. The mathematical findings have discovered that four or three items in each chunk is the most optimal.[15]"]])
     chunker.set_data(table)

     # Création d'une boucle d'attente pour simuler un chargement bloquant
     loop = QEventLoop( )
     # Connecte la fin du chargement pour quitter la boucle
     def on_finish():
         loop.quit()
     chunker.thread.finish.connect(on_finish)
     loop.exec_()
     chunker.Outputs.data.send = receiver.receive_data
     chunker.Outputs.data.send(chunker.result)
     data = receiver.received_data
     for line in data:
         print(line["Chunks"].value)
     if len(data[0]["Chunks"].value) > 0 :
             return 0
     else:
        return 1

def check_widget_chunker():
     app = QApplication(sys.argv)
     receiver = SignalReceiver()
     chunker = OWChunker()
     if check_create_chunker(chunker, receiver) != 0:
           print("Erreur à la création des embeddings")
           return 1
     return 0


if __name__ == "__main__":
     check_widget_chunker()


import os
import time
if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils.MetManagement import getTempDir
else:
    from orangecontrib.AAIT.utils.MetManagement import getTempDir
"""
only windows!!!
when you launch GPT4all without an interface it is not possible to simply close it, this file is intended to stop it 
when it has not been used for a long time (1hour)

"""


def exit_chat_exe():
    """
    send a command line to stop gpt4all process
    """
    ldc = '"taskkill /im chat.exe /t /f"'
    try:
        os.system(ldc)
    except Exception as e:
        print(e)
        print(e)


def need_to_quit(delta_time_in_second):
    """
    read a file in temp folder. this file contains the date and time of the last call to GPT4ALL
    if it has been too long since the last call was made, the process is stopped
    """
    # Obtenir le chemin du répertoire temp de Windows
    temp_dir = getTempDir()

    # Créer le nom complet du fichier
    file_path = os.path.join(temp_dir, 'date_heure.txt')

    try:
        with open(file_path, 'r') as file:
            stored_time_seconds = int(file.read())

        current_time_seconds = int(time.time())
        time_difference = abs(current_time_seconds - stored_time_seconds)
        print(f"Temps écoulé depuis la dernière écriture : {time_difference} secondes")
        if time_difference < delta_time_in_second:
            return False
    except FileNotFoundError:
        print("File date_heure.txt not found")

        ## si le fichier n'existe pas on quitte pour éviter la fermeture si l'utisateur veut se servir de 4all
        return True
    return True


if os.name != 'nt':
    print("only windows -> exiting")
    exit(0)

print("debut attente")
time.sleep(30)
while True:
    # si horodatage non mis à jour au bout 1h on kill and quit
    if need_to_quit(3600) == True:
        print("need to quit")
        exit_chat_exe()
        exit(0)
    time.sleep(10)
print("fin attente")



import os
import sys

import Orange.data
from AnyQt.QtWidgets import QApplication
from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output
from orangecontrib.text.corpus import Corpus


if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
    from Orange.widgets.orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
else:
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file

@apply_modification_from_python_file(filepath_original_widget=__file__)
class OWTable2Corpus(widget.OWWidget):
    name = "Table to Corpus"
    description = "Convert a Table to a Corpus and set the column 'content' as the used text feature."
    icon = "icons/owtable2corpus.svg"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/owtable2corpus.svg"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owtable2corpus.ui")
    want_control_area = False
    priority = 1060

    class Inputs:
        data = Input("Data", Orange.data.Table)

    class Outputs:
        data = Output("Corpus", Corpus)

    @Inputs.data
    def set_data(self, in_data):
        self.data = in_data
        if self.autorun:
            self.run()


    def __init__(self):
        super().__init__()
        # Qt Management
        self.setFixedWidth(470)
        self.setFixedHeight(300)
        uic.loadUi(self.gui, self)

        # Data Management
        self.data = None
        self.autorun = True
        self.post_initialized()

    def run(self):
        self.warning("")
        if self.data is None:
            return

        if not "content" in self.data.domain:
            self.warning("You haven't defined a 'content' variable.")

        domain = self.data.domain
        out_data = Corpus.from_table(domain, self.data)
        out_data.text_features = [Orange.data.StringVariable(name="content")]
        self.Outputs.data.send(out_data)

    def post_initialized(self):
        pass

if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = OWTable2Corpus()
    my_widget.show()
    app.exec_()



import os
import sys
import time
import requests
import orangecanvas.application.canvasmain as canvasmain
import Orange.data
from AnyQt.QtWidgets import QApplication, QLabel
from Orange.widgets import widget, gui
from Orange.widgets.utils.signals import Input, Output
from Orange.widgets.settings import Setting
from AnyQt.QtWidgets import QLineEdit, QTextBrowser, QTextEdit

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.llm import answers
    from Orange.widgets.orangecontrib.AAIT.utils import thread_management
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
    from Orange.widgets.orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
else:
    from orangecontrib.AAIT.llm import answers
    from orangecontrib.AAIT.utils import thread_management
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file

@apply_modification_from_python_file(filepath_original_widget=__file__)
class OWDeep_Search(widget.OWWidget):
    name = "Deep Search"
    description = "Generate a response to a column 'prompt' with a LLM"
    icon = "icons/owqueryllm.svg"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/owqueryllm.svg"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owdeepsearch.ui")
    want_control_area = True
    priority = 1089

    class Inputs:
        api_key = Input("API KEY", Orange.data.Table)
        prompt = Input("Prompt", Orange.data.Table)

    class Outputs:
        response = Output("Response", Orange.data.Table)

    @Inputs.api_key
    def set_api_key(self, in_api_key: Orange.data.Table):
        if in_api_key is not None:
            if "api_key" not in in_api_key.domain:
                self.error("You need a column api_key");
                return
            self.api_key = str(in_api_key[0]["api_key"])  # ← cast en str
            self.line_api_key.setText(self.api_key)
            self.error(None)
            if self.autorun:
                self.run()


    @Inputs.prompt
    def set_prompt(self, in_prompt: Orange.data.Table):
        if in_prompt is not None:
            if "prompt" not in in_prompt.domain:
                self.error("You need a column prompt");
                return
            self.prompt = str(in_prompt[0]["prompt"])  # ← cast en str
            self.error(None)
            if self.autorun:
                self.run()

    def __init__(self):
        super().__init__()
        # Qt Management
        self.setFixedWidth(700)
        self.setFixedHeight(500)
        uic.loadUi(self.gui, self)
        self.label_description = self.findChild(QLabel, 'Description')
        # canvasmain.CanvasMainWindow.set_signal_freeze(self, True)
        # print(self.label_description.text())
        self.line_api_key: QLineEdit = self.findChild(QLineEdit, "line_api_key")
        # Connexion du champ à la fonction de mise à jour
        self.line_api_key.editingFinished.connect(self.update_api_key)
        # Text browser
        self.textBrowser = self.findChild(QTextBrowser, 'textBrowser')
        # Data Management
        self.api_key = None
        self.prompt = None
        self.thread = None
        self.autorun = True
        self.can_run = True
        self.result = None  # sera ma table de sortie
        # Custom updates
        self.post_initialized()

    def update_api_key(self):
        self.api_key = self.line_api_key.text().strip()

    def deepsearch(self, api_key: str, prompt: str):
        """
        Appelle Perplexity Sonar Deep-Research et renvoie la réponse complète
        dans une Orange Table (métadonnée 'response').
        """

        url = "https://api.perplexity.ai/chat/completions"
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {api_key}"
        }
        payload = {
            "model": "sonar-deep-research",
            "messages": [
                {"role": "system",
                 "content": ("You are a deep research assistant. "
                             "Answer only in Markdown without exposing any chain-of-thought.")},
                {"role": "user", "content": prompt}
            ],
            "temperature": 0.0,
            "max_tokens": 20_000,
            "format": "markdown"
        }

        try:
            resp = requests.post(url, json=payload, headers=headers, timeout=600)
            resp.raise_for_status()
            answer_md = resp.json()["choices"][0]["message"]["content"]

        except requests.HTTPError as e:
            print("STATUS :", resp.status_code)  # affiche 400, 401, etc.
            print("DETAIL :", resp.text)  # message JSON complet de Perplexity

            answer_md = f"**Erreur Perplexity :** {resp.text}"

        except Exception as e:
            answer_md = f"**Erreur Perplexity :** {e}"

        # Affiche la réponse complète dans la console
        # print("Réponse Perplexity :\n", answer_md)

        response_var = Orange.data.StringVariable("response")

        domain = Orange.data.Domain([], metas=[response_var])
        out_data = Orange.data.Table.from_list(domain, [[answer_md]])
        #self.Outputs.response.send(out_data)
        return out_data

    def run(self):
        # print("I'm running...")
        # print(self.prompt)
        # print(self.api_key)
        if not self.can_run:
            return

        if self.prompt == None:
            return

        if self.api_key == None:
            # print("Je sors")
            return

        # If Thread is already running, interrupt it
        if self.thread is not None:
            if self.thread.isRunning():
                self.thread.safe_quit()

        # Start progress bar
        self.progressBarInit()
        #◙self.textBrowser.setText("")

        # Connect and start thread : main function, progress, result and finish
        # --> progress is used in the main function to track progress (with a callback)
        # --> result is used to collect the result from main function
        # --> finish is just an empty signal to indicate that the thread is finished
        self.thread = thread_management.Thread(self.deepsearch, self.api_key, self.prompt)
        self.thread.progress.connect(self.handle_progress)
        self.thread.result.connect(self.handle_result)
        self.thread.finish.connect(self.handle_finish)
        self.thread.start()

    def handle_progress(self, progress) -> None:
        value = progress[0]
        text = progress[1]
        if value is not None:
            self.progressBarSet(value)
        if text is None:
            self.textBrowser.setText("")
        else:
            self.textBrowser.insertPlainText(text)

    def handle_result(self, result):
        try:
            self.result = result
            self.Outputs.response.send(result)
            # Orange.canvas.freeze(True)
        except Exception as e:
            print("An error occurred when sending out_data:", e)
            self.Outputs.response.send(None)
            return

    def handle_finish(self):
        print("Generation finished")
        self.progressBarFinished()


    def post_initialized(self):
        pass


if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = OWDeep_Search()
    my_widget.show()
    app.exec_()




import os
import sys
import re
import Orange.data
from Orange.widgets import widget
from Orange.widgets.widget import Input, Output
from Orange.data import Table, Domain,  DiscreteVariable
from AnyQt.QtWidgets import QPushButton, QCheckBox
from Orange.widgets.settings import Setting
# from fontTools.varLib.mutator import curr

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
    from Orange.widgets.orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
else:
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file

@apply_modification_from_python_file(filepath_original_widget=__file__)
class OWApplyRules(widget.OWWidget):
    name = "Apply Rules to data"
    description = "Apply Rules to data fron an over workflow."
    icon = "icons/apply_rules.svg"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/apply_rules.svg"
    priority = 1145
    keywords = "Apply Rules to data"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owapplyrules.ui")
    want_control_area = False
    class Inputs:
        rules = Input("Rules", Orange.data.Table)
        data = Input("Data", Orange.data.Table)

    class Outputs:
        out_data = Output("Out data", Orange.data.Table)

    strauto: str = Setting('False')


    @Inputs.rules
    def set_rules(self, data):
        self.rules_data = data
        self.Outputs.out_data.send(None)
        if self.strauto == 'True' or self.strauto == 'Middle':
            self.run()
            if self.strauto == 'Middle':
                self.strauto = 'False'
                self.checkbox_interface.setChecked(False)

    @Inputs.data
    def set_new_point_from_scatter_plot(self, data):
        self.data = data
        self.Outputs.out_data.send(None)
        if self.strauto == 'True' or self.strauto == 'Middle':
            self.run()
            if self.strauto == 'Middle':
                self.strauto = 'False'
                self.checkbox_interface.setChecked(False)

    def __init__(self):
        super().__init__()
        # Set the fixed width and height of the widget
        self.setFixedWidth(470)
        self.setFixedHeight(300)

        # Load the user interface file
        uic.loadUi(self.gui, self)
        self.rules_data = None
        self.data = None
        self.btn_run = self.findChild(QPushButton, "pushButton")
        self.btn_run.clicked.connect(self.run)


        self.checkbox_interface = self.findChild(QCheckBox, 'checkBox')
        if self.strauto == 'Middle':
            self.strauto = 'False'
        if self.strauto == 'False':
            self.checkbox_interface.setChecked(False)
        if self.strauto == 'True':
            self.checkbox_interface.setChecked(True)
        self.checkbox_interface.stateChanged.connect(self.on_checkbox_toggled)

        self.post_initialized()

    def post_initialized(self):
        """
        used for overloading only
        """
        return

    def del_space_debut_fin(self, text_to_edit):
        if text_to_edit[0] == " ":
            text_to_edit = text_to_edit[1:]
        if text_to_edit[-1] == " ":
            text_to_edit = text_to_edit[:-1]
        return text_to_edit



    def run(self):
        self.error("")
        self.warning("")
        if self.rules_data is None or self.data is None:
            self.error("You must have rules and data")
            return



        data_regle = self.rules_data
        data_value = self.data
        num_col_regle = data_regle.domain.index("regle")
        a_recuperer = []
        var_not_in_domain = []
        value_in_rule_not_in_domain = []
        error = False
        for i in range(len(data_value)):
            a_recuperer.append(False)
        for regle in data_regle:
            current_regle = regle[num_col_regle]

            ## cette partie permet de gérer le cas ou dans une règle une variable n'est pas présente dans les données d'entrées et de pas avoir une erreur
            regl_list = str(current_regle).split(" and ")
            compteur=0
            for unit_rule in regl_list:
                compteur=compteur+1
                current_var, current_symb, current_value = re.split(r'(<=|>=)', unit_rule)
                current_var = self.del_space_debut_fin(current_var)
                if current_var not in data_value.domain:
                    var_not_in_domain.append(current_var)
                    if compteur!=1:
                        current_var="and "+ current_var
                    value_in_rule_not_in_domain.append(current_var +" "+ current_symb + current_value)
            if len(var_not_in_domain) > 0:
                self.warning("Warning " + ",".join(var_not_in_domain) + " not in possible variable")

                for j,element in enumerate(value_in_rule_not_in_domain):
                    current_regle = str(current_regle).replace(element, "")

            # if current rules start with "     and " we need to remove the begin of the rule
            test_current_regle=str(current_regle)
            # print("#####")
            # print(test_current_regle)
            # print(type(test_current_regle))
            # print("##########################")
            while(test_current_regle[0]==" "):
                test_current_regle=test_current_regle[1:]
            if len(test_current_regle)>=4:
                if test_current_regle[:4]=="and ":
                    test_current_regle=test_current_regle[4:]
                    current_regle=test_current_regle
                    print(current_regle)
            for i in range(len(data_value)):
                new_regle = str(current_regle)

                for j in range(len(data_value[i])):
                    new_regle = new_regle.replace(data_value.domain[j].name, str(data_value[i][j].value))
                a_recuperer[i] = 0

                try:
                    if eval(new_regle):
                        a_recuperer[i] = 1
                except Exception as e:
                    error = True
                    print(f"Error : {e}")
                    self.error("You have to edit your rule to be correct. Check meta in input data too!")
        data = []
        for idx, element in enumerate(a_recuperer):
            d = []
            for i, elem in enumerate(data_value[idx]):
                d.append(elem)
            d.append(str(int(element)))
            data.append(d)

        # Construction du nouveau domaine
        domain_attrs = list(data_value.domain.attributes)  # Récupération des attributs
        target_var = DiscreteVariable("ok", values=["0", "1"])  # Variable cible
        target_var.colors = [(255, 0, 0),  # Rouge foncé (Dark Red) pour "0"
                             (0, 255, 0)]  # Vert foncé (Dark Green) pour "1"

        # Gestion des métadonnées (si présentes)
        meta_vars = list(data_value.domain.metas) if data_value.domain.metas else []

        # Création du nouveau domaine avec les métadonnées incluses
        new_domain = Domain(domain_attrs, target_var, metas=meta_vars)

        # Création de la table avec les nouvelles données
        out_data = Table.from_list(new_domain, data)

        # Récupération et réintégration des métadonnées
        if meta_vars:
            out_data.metas = data_value.metas  # Copie des métadonnées depuis l'entrée

        # Envoi des données si pas d'erreur
        if not error:
            self.Outputs.out_data.send(out_data)

    def on_checkbox_toggled(self, state):
        if state == 2:  # Qt.Checked (valeur 2 pour "coché")
            self.strauto='True'
        elif state == 0:  # Qt.Unchecked (valeur 0 pour "décoché")
            self.strauto='False'
        elif state==1:
            self.strauto = 'Middle'

if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication
    app = QApplication(sys.argv)
    my_widget = OWApplyRules()
    my_widget.show()
    app.exec_()



import os
import sys

from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output
from Orange.data import Table, Domain, StringVariable
from langdetect import detect, LangDetectException
from AnyQt.QtWidgets import QApplication

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
    from Orange.widgets.orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
else:
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file

@apply_modification_from_python_file(filepath_original_widget=__file__)
class OWLanguageDetection(widget.OWWidget):
    name = "Language Detection"
    description = "Detects language of text content."
    icon = "icons/languages.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/languages.png"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owlangdetect.ui")
    want_control_area = False
    priority = 1060

    class Inputs:
        data = Input("Content", Table)

    class Outputs:
        data = Output("Content with language", Table)

    def __init__(self):
        super().__init__()
        self.data = None
        self.autorun = True
        self.setFixedWidth(470)
        self.setFixedHeight(300)
        uic.loadUi(self.gui, self)
        self.autorun = True

    @Inputs.data
    def set_data(self, in_data):
        self.data = in_data
        if self.autorun and self.data:
            self.process()

    def detect_language(self, text):
        try:
            return detect(text)
        except LangDetectException:
            return "unknown"

    def process(self):
        if self.data is None:
            print("[INFO] No input data provided.")
            return

        if "Content" not in self.data.domain:
            self.error("Missing 'Content' column in input data")
            print("[ERROR] 'Content' column not found in input data.")
            return

        self.error("")

        language_var = StringVariable("Language")
        new_domain = Domain(self.data.domain.attributes,
                            self.data.domain.class_vars,
                            self.data.domain.metas + (language_var,))

        new_metas = []
        for i, row in enumerate(self.data):
            text = str(row["Content"])
            lang = self.detect_language(text)
            print(f"[DEBUG] Row {i}: Detected language -> {lang}")
            new_metas.append(list(row.metas) + [lang])

        out_data = Table(new_domain, self.data.X, self.data.Y, new_metas)
        print("[INFO] Language detection complete. Sending output data.")
        self.Outputs.data.send(out_data)

if __name__ == "__main__":
    app = QApplication(sys.argv)
    ow = OWLanguageDetection()
    ow.show()
    app.exec_()



# Currently active OWS
current_ows = "toto"

def get_current_ows():
    return current_ows
# Currently active OWS id
ptr_current_canvas_main = 0
# List of all OWS
vect_doc_ptr = []  # [path, id, first opening time]

# Vector containing the master external in the order of invocation
# Value is "cmd_line" if it's from the command line, otherwise it's the ID
ptr_master_external_orange = []



import os
import sys
import ntpath
import platform

import Orange.data
from AnyQt.QtWidgets import QApplication, QLabel
from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output
from AnyQt.QtWidgets import QCheckBox
from Orange.widgets.settings import Setting


if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.llm import GPT4ALL
    from Orange.widgets.orangecontrib.AAIT.utils import  thread_management
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
    from Orange.widgets.orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
    from Orange.widgets.orangecontrib.AAIT.utils.MetManagement import getTempDir
else:
    from orangecontrib.AAIT.llm import GPT4ALL
    from orangecontrib.AAIT.utils import  thread_management
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
    from orangecontrib.AAIT.utils.MetManagement import getTempDir





@apply_modification_from_python_file(filepath_original_widget=__file__)
class OWLLM4ALL(widget.OWWidget):
    name = "LLM Local 4 All"
    description = "Query a local LLM to get a response (deprecated -> use query llm instead)"
    icon = "icons/llm4all.svg"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/llm4all.svg"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owllm4all.ui")
    want_control_area = False
    priority = 1090
    class Inputs:
        data = Input("Data", Orange.data.Table)
        model = Input("Model", str, auto_summary=False)

    class Outputs:
        data = Output("Data", Orange.data.Table)

    stropen4all: str = Setting('False')  # ty
    @Inputs.data
    def set_data(self, in_data):
        """
        Setter for the input data.

        Parameters
        ----------
        in_data : Orange.data.Table
            The input table.

        Returns
        -------
        None

        """
        if not self.runable:
            return
        self.error("")
        # Set the input data
        self.data = in_data
        if self.data is None:
            return
        if "prompt" not in self.data.domain:
            self.error("input table need a prompt column")
            return
        if self.model==None:
            return
        # Run the widget
        self.run()
    def post_initialized(self):
        """
        used for overloading only
        """
        return

    @Inputs.model
    def set_model(self, model):
        """
        Setter for the model.

        Parameters
        ----------
        model : str
            The model.

        Returns
        -------
        None
        """
        if not self.runable:
            return
        self.error("")
        # Set the model
        self.model = ntpath.basename(model)
        if not self.model.endswith(".gguf"):
            self.error("Model needs to be in a gguf format.")
            return
        if self.model is None:
            return
        if self.data is None:
            self.error("widget need an input data with a prompt column")
            return
        if "prompt" not in self.data.domain:
            self.error("input table need a prompt column")
            return

        # Run the widget
        self.run()
    def __init__(self):
        """
        Initialize the widget.

        This function initializes the widget and sets up its basic properties.
        It also loads the user interface file and finds the label for the description.
        """

        super().__init__()
        self.runable=True
        # Initialize path management
        # This is used to store the current Orange Widgets (OWS) path
        self.current_ows = ""

        # Set the fixed width and height of the widget
        self.setFixedWidth(470)
        self.setFixedHeight(300)

        # Load the user interface file
        uic.loadUi(self.gui, self)

        # Find the label for the description
        self.label_description = self.findChild(QLabel, 'Description')
        self.checkbox_interface= self.findChild(QCheckBox, 'checkBox')
        if self.stropen4all == 'Coucicouca':
            self.stropen4all = 'False'
        if self.stropen4all=='False':
            self.checkbox_interface.setChecked(False)
        if self.stropen4all == 'True':
            self.checkbox_interface.setChecked(True)

        self.checkbox_interface.stateChanged.connect(self.on_checkbox_toggled)

        # Initialize data management
        # This is used to store the input data
        self.data = None
        self.model = None#"solar-10.7b-instruct-v1.0.Q6_K.gguf"
        self.result = None
        # Initialize thread management
        # This is used to handle the background thread
        self.thread = None
        self.post_initialized()

    def on_checkbox_toggled(self, state):
        if state == 2:  # Qt.Checked (valeur 2 pour "coché")
            self.stropen4all='True'
        elif state == 0:  # Qt.Unchecked (valeur 0 pour "décoché")
            self.stropen4all='False'
        elif state==1:
            self.stropen4all = 'Coucicouca'


    def run(self):
        """
        Run the widget.

        This function runs the widget by initializing the thread and starting it.
        It also handles the case where the thread is already running and interrupts it.

        Returns:
            None
        """
        if not self.runable:
            return
        # Clear the error message
        self.error("")


        chemin_fichier = os.path.join(getTempDir(), 'view4all.txt')
        if self.stropen4all=='False':
            # Vérifier si le fichier existe
            if os.path.exists(chemin_fichier):
                try:
                    # Supprimer le fichier
                    os.remove(chemin_fichier)
                except Exception as e:
                    print(f"Error : {e}")
            else:
                print(f"Le fichier n'existe pas : {chemin_fichier}")
        if self.stropen4all == 'True' or self.stropen4all == 'Coucicouca':
            if False==os.path.exists(chemin_fichier):
                try:
                    # Créer et écrire dans le fichier
                    with open(chemin_fichier, 'w') as fichier:
                        fichier.write("")
                except Exception as e:
                    print(f"Error : {e}")


        # If Thread is already running, interrupt it
        #if self.thread is not None:
        #    if self.thread.isRunning():
        #        self.thread.safe_quit()

        # If data is not provided, exit the function
        if self.data is None:
            return

        # Start progress bar
        self.progressBarInit()
        if os.name != "nt":
            self.warning("Your OS isn't Windows, please configure GPT4ALL to run as server")


        # Connect and start thread : main function, progress, result and finish
        # --> progress is used in the main function to track progress (with a callback)
        # --> result is used to collect the result from main function
        # --> finish is just an empty signal to indicate that the thread is finished
        #GPT4ALL.open_gpt_4_all(self.data)
        self.thread = thread_management.Thread(GPT4ALL.open_gpt_4_all, self.data, self.model, widget=self)
        self.thread.progress.connect(self.handle_progress)
        self.thread.result.connect(self.handle_result)
        self.thread.finish.connect(self.handle_finish)
        self.thread.start()

    @staticmethod
    def set_4allpath(path_name):
        """
        you can call this function to set specifical path of 4all
        """
        GPT4ALL.gpt4all_path=path_name

    def handle_progress(self, value: float) -> None:
        """
        Sets the progress bar value to the given value.

        Args:
            value (float): The value to set the progress bar to.

        Returns:
            None
        """
        # Set the progress bar value to the given value

        # The value parameter is a float between 0 and 1, representing
        # the current progress of the operation.
        self.progressBarSet(value)

    def handle_result(self, result):
        """
        Handles the result of the main function.

        Sends the result to the output data port if there are no errors.
        If there is an error, sends None to the output data port and prints
        the error message.

        Args:
            result (Any): The result of the main function.

        Returns:
            None
        """
        try:
            # Send the result to the output data port
            self.Outputs.data.send(result)
            self.result = result
        except Exception as e:
            # If there is an error, send None to the output data port
            # and print the error message
            print("An error occurred when sending out_data:", e)
            self.Outputs.data.send(None)
            return

    def handle_finish(self):
        """
        Handles the finish signal of the main function.

        Prints a message indicating that the generation is finished and
        updates the progress bar to be finished.

        Returns:
            None
        """
        # Print a message indicating that the generation is finished
        print("Generation finished")
        # Set the progress bar to be finished
        self.progressBarFinished()



if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = OWLLM4ALL()
    my_widget.show()
    app.exec_()



import subprocess
import sys
from pathlib import Path
import os
import time

# Global Path and Timeout Definitions
widget_path = Path(__file__).parent.parent / "widgets"
OVERALL_TIMEOUT_DURATION = 60  # Timeout limit for all widget executions combined

# Global dictionary to collect results
results_summary = {}

def process_widget_output(widget, return_code, stderr_output):
    """
    Processes the error output from a widget execution.

    Parameters:
    widget (Path): Path object pointing to the widget script.
    return_code (int): The subprocess return code.
    stderr_output (str): Complete stderr data as a UTF-8 string.
    """
    global results_summary
    stderr_output = stderr_output.strip()
    contains_warning = 'warning' in stderr_output.lower()
    widget_name = widget.name

    if return_code is None:
        results_summary[widget_name] = "In Progress"
    elif return_code == 0:
        if stderr_output:
            if contains_warning:
                results_summary[widget_name] = "Warning"
            else:
                results_summary[widget_name] = "No Output"
        else:
            results_summary[widget_name] = "Success"
    else:
        error_message = f"Error Code {return_code}:\n{stderr_output}"
        if contains_warning:
            results_summary[widget_name] = f"Warning with {error_message}"
        else:
            results_summary[widget_name] = error_message

def execute_widget_as_module(widget):
    """
    Executes a single Python widget script as a module with the -m flag.

    Parameters:
    widget (Path): Path object pointing to the widget script to be run.

    Returns:
    tuple: A combination of the widget path and the initiated subprocess.
    """
    module_name = '.'.join(widget.with_suffix('').relative_to(Path(__file__).parent.parent).parts)
    
    # Execute the widget script using unbuffered output
    process = subprocess.Popen(
        [sys.executable, '-m', module_name],
        cwd=widget_path.parent,
        stdout=subprocess.DEVNULL,
        stderr=subprocess.PIPE,
        env=dict(os.environ, PYTHONUNBUFFERED="1")
    )
    
    return widget, process

def test_all_widgets():
    """
    Executes all Python scripts in the widgets directory as modules in parallel,
    excluding '__init__.py', within a specified total execution time limit.
    """
    processes = []

    # Launch all widget processes
    for widget in widget_path.glob("*.py"):
        if widget.name != "__init__.py":
            processes.append(execute_widget_as_module(widget))
    
    # Allow all processes to run for a maximum of 60 seconds
    time.sleep(OVERALL_TIMEOUT_DURATION)

    # Collect outputs after the timeout period
    for widget, process in processes:
        if process.poll() is None:
            process.terminate()  # Terminate if still running
            process.wait()  # Ensure process has terminated properly
            process_widget_output(widget, None, "")
        else:
            _, stderr_data = process.communicate()
            stderr_output = stderr_data.decode('utf-8')
            process_widget_output(widget, process.returncode, stderr_output)

def main():
    """
    Main function to trigger widget testing and output a summary with error details.
    """
    test_all_widgets()

    # Print a separated summary of errors and warnings
    print("\n___ SUMMARY OF ERRORS AND WARNINGS ___")

    # Display only the widgets with errors or warnings
    any_issues = False
    for widget_name, issue in results_summary.items():
        if issue not in ["In Progress", "Success", "No Output"]:
            print(f"{widget_name}: {issue}")
            any_issues = True

    # Check if there were any issues; if not, print a success message
    if not any_issues:
        print("All widgets executed successfully or are in progress with no issues.")
    else:
        raise Exception("One or more widgets encountered issues during execution.")

if __name__ == "__main__":
    main()



import os
import sys
import Orange.data
from Orange.data import Domain, Table,  ContinuousVariable
from Orange.widgets.utils.signals import Input, Output
from Orange.widgets.widget import Output, OWWidget
import numpy as np

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
else:
    from orangecontrib.AAIT.utils.import_uic import uic


class LoopStartWidget(OWWidget):
    name = "Loop Start"
    description = "Widget to start a loop with data table input and output."
    icon = "icons/startloop.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/startloop.png"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owstartloop.ui")
    want_control_area = False
    priority = 1010

    class Inputs:
        data_in = Input("Data In", Orange.data.Table)
        # in_pointer = Input("End of the Loop Do-While", str, auto_summary=False)

    class Outputs:
        data_out = Output("Data Out", Orange.data.Table)
        out_pointer = Output("Begin of the Loop Do-While", str, auto_summary=False)

    def __init__(self):
        super().__init__()

        self.setFixedWidth(470)
        self.setFixedHeight(300)
        uic.loadUi(self.gui, self)
        self.data = None
        self.send_pointer()


    @Inputs.data_in
    def set_data(self, dataset):
        if dataset is None:
            print("No data received.")
            return
        self.error("")
        self.data=None
        error_code_check_iter_column=self.check_iter_column(dataset)
        if error_code_check_iter_column==1:
            print("add iter column")
            self.data=self.add_iter_column(dataset)
        elif error_code_check_iter_column==0:
            self.data = dataset
        else:
            self.error("error remove iter column in input data!!!")
            return
        self.process_data()

    def add_iter_column(self,in_data_orange):
        iter_var = ContinuousVariable("iter")
        new_domain = Domain(in_data_orange.domain.attributes + (iter_var,), in_data_orange.domain.class_var, in_data_orange.domain.metas)
        iter_values = np.ones((len(in_data_orange), 1))  # column of 0
        # merge
        new_X = np.hstack((in_data_orange.X, iter_values))
        return Table(new_domain, new_X, in_data_orange.Y, in_data_orange.metas)

    def check_iter_column(self,in_data_orange):
        """
        Checks the presence and validity of the categorical variable 'iter' in in_data.

        Returns:
        - 0 if 'iter' is a categorical variable with only "0" and "1" as values.
        - 1 if 'iter' is missing from the domain.
        - 2 if 'iter' exists but is either not continuous.
        """
        # Check if "iter" exists in the domain
        if "iter" not in in_data_orange.domain:
            return 1  # "iter" does not exist

        iter_var = in_data_orange.domain["iter"]

        # Check if "iter" is continuous
        if not iter_var.is_continuous:
            return 2  # "iter" is not continuous


        return 0  # Everything is correct


    def get_nb_line(self):
        """Return the number of lines to be called from another widget."""
        return 0 if self.data is None else len(self.data)

    def get_column_name_and_type(self):
        """Return the name and type of 'data_in' to be called from another widget."""
        if self.data is None:
            return [[], []]
        column_names = []
        column_types = []

        for element in self.data.domain.variables + self.data.domain.metas:
            column_names.append(str(element.name))
            column_types.append(str(type(element)))
        return column_names, column_types
    def get_in_data(self):
        return self.data


    def process_data(self):
        """Main process executed when data is available."""
        if self.data is not None:
            #new_data=copy.deepcopy(self.data)
            self.Outputs.data_out.send(self.data)  # Envoie les données en sortie
        else:
            print("No data sent.")


    def send_pointer(self):
        """Send a pointer to the current class for the loop."""
        pointer = str(id(self))
        self.Outputs.out_pointer.send(pointer)

if __name__ == "__main__":
    from AnyQt.QtWidgets import QApplication

    app = QApplication(sys.argv)
    obj = LoopStartWidget()
    obj.show()
    app.exec_()



import sys
import os

import Orange.data
from AnyQt.QtWidgets import QApplication
from Orange.widgets import widget
from Orange.widgets.widget import  MultiInput, Output
from Orange.widgets.utils.signals import Input, Output
from Orange.data import Table


if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
else:
    from orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file


@apply_modification_from_python_file(filepath_original_widget=__file__)
class POW_Wfactory(widget.OWWidget):
    name = "Widget Factory"
    description = "Simply create widget"
    icon = "icons/widgetFactory.svg"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/widgetFactory.svg"
    priority = 1900

    # to  be continued
    class Inputs:
        data = MultiInput(
            "Data", Table, replaces=["in_data"], default=True
        )
        object = MultiInput(
            "Object", object, replaces=["in_object"], default=False, auto_summary=False
        )

    class Outputs:
        data_out = Output("Data Out", Orange.data.Table)

    signal_names = ("data")

    def post_initialized(self):
        return

    def __init__(self):
        super().__init__()
        self.in_data = None
        self.out_data = None
        self.in_datas = None
        self.in_object = None
        self.in_objects = None
        self.in_learner = None
        self.in_learners = None
        self.in_classifier = None
        self.in_classifiers = None
        self.out_datas = None
        self.out_object = None
        self.out_objects = None
        self.out_learner = None
        self.out_learners = None
        self.out_classifier = None
        self.out_classifiers = None
        self.post_initialized()

        for name in self.signal_names:
            setattr(self, name, [])

    def remove_input(self, index, signal):
        if signal == "data":
            data = []
            if self.in_data is not None:
                self.in_data = None
            else:
                data = self.in_datas
                data.pop(index)
                self.in_datas = None
            if len(data) == 1:
                self.in_data = data[0]
            if len(data) > 1:
                self.in_datas = data
        if signal == "object":
            objects = []
            if self.in_object is not None:
                self.in_object = None
            else:
                objects = self.in_objects
                objects.pop(index)
                self.in_objects = None
            if len(objects) == 1:
                self.in_object = objects[0]
            if len(objects) > 1:
                self.in_objects = objects
        self.run()

    @Inputs.data
    def set_data(self, index, data):
        self.in_datas[index] = data
        self.run()

    @Inputs.data.insert
    def insert_data(self, index, data):
        if index == 0:
            self.in_data = data
        else:
            dic = []
            if index == 1:
                dic.append(self.in_data)
                self.in_data = None
            if index > 1:
                dic = self.in_datas
            dic.append(data)
            self.in_datas = dic
        self.run()

    @Inputs.data.remove
    def remove_data(self, index):
        self.remove_input(index, "data")


    @Inputs.object
    def set_object(self, index, object):
        self.set_input(index, object, "object")

    @Inputs.object.insert
    def insert_object(self, index, object):
        if index == 0:
            self.in_object = object
        else:
            dic = []
            if index == 1:
                dic.append(self.in_object)
                self.in_object = None
            if index > 1:
                dic = self.in_objects
            dic.append(object)
            self.in_objects = dic
        self.run()

    @Inputs.object.remove
    def remove_object(self, index):
        self.remove_input(index, "object")

    def run(self):
        self.module_kernel_run()
        if self.out_data != None:
            self.Outputs.data_out.send(self.out_data)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = POW_Wfactory()
    my_widget.show()
    app.exec()


import json
import Orange


def parse_tab_file_orange(file_path):
    """
    Lit un fichier .tab avec Orange et renvoie la liste d'infos sur les variables.
    Chaque élément de cette liste est un dict du style:
    {
        "name": <str>,
        "type": <"continuous"|"categorical"|"string">,
        "categories": <list(str)|None>,
        "role": <"feature"|"target"|"meta">
    }
    """
    data = Orange.data.Table(file_path)
    domain = data.domain

    domain_info = []

    # Parcourir les attributs (features)
    for feature in domain.attributes:
        domain_info.append({
            "name": feature.name,
            "type": ("continuous" if isinstance(feature, Orange.data.ContinuousVariable)
                     else "categorical" if isinstance(feature, Orange.data.DiscreteVariable)
            else "string"),
            "categories": feature.values if hasattr(feature, "values") else None,
            "role": "feature"
        })

    # Parcourir les cibles (targets)
    for target in domain.class_vars:
        domain_info.append({
            "name": target.name,
            "type": ("continuous" if isinstance(target, Orange.data.ContinuousVariable)
                     else "categorical" if isinstance(target, Orange.data.DiscreteVariable)
            else "string"),
            "categories": target.values if hasattr(target, "values") else None,
            "role": "target"
        })

    # Parcourir les métadonnées (meta)
    for meta_var in domain.metas:
        domain_info.append({
            "name": meta_var.name,
            "type": ("continuous" if isinstance(meta_var, Orange.data.ContinuousVariable)
                     else "categorical" if isinstance(meta_var, Orange.data.DiscreteVariable)
            else "string"),
            "categories": meta_var.values if hasattr(meta_var, "values") else None,
            "role": "meta"
        })
    print(domain_info)
    return domain_info


def domain_info_to_json(domain_info):
    """
    Convertit la liste de dictionnaires (issue de parse_tab_file_orange)
    en une chaîne JSON (str).
    """
    print(json.dumps(domain_info, ensure_ascii=False, indent=4))
    return json.dumps(domain_info, ensure_ascii=False, indent=4)


def compare_domain_info(domain_info1, domain_info2):
    """
    Compare deux listes de variables (domain_info1 et domain_info2).
    Vérifie que, pour chaque variable (même 'name'),
      le type, la liste de catégories, et le rôle sont identiques.
    Ignore l'ordre des variables.

    Retourne:
      - (True, "") si tout est identique
      - (False, "Message d'erreur") sinon
    """

    # Transformer domain_info2 en dict indexé par le name
    dict_info2 = {v["name"]: v for v in domain_info2}

    # 1) Vérifier que chaque variable de domain_info1 est dans domain_info2
    for var1 in domain_info1:
        name = var1["name"]
        if name not in dict_info2:
            return (False, f"La variable '{name}' est présente dans le premier JSON mais pas dans le second.")

        var2 = dict_info2[name]

        # Comparer le type
        if var1["type"] != var2["type"]:
            return (False, f"La variable '{name}' a un type différent: "
                           f"{var1['type']} vs {var2['type']}.")

        # Comparer le rôle
        if var1["role"] != var2["role"]:
            return (False, f"La variable '{name}' a un rôle différent: "
                           f"{var1['role']} vs {var2['role']}.")

        # Comparer les catégories (uniquement si type = 'categorical')
        if var1["type"] == "categorical":
            set1 = set(var1["categories"]) if var1["categories"] else set()
            set2 = set(var2["categories"]) if var2["categories"] else set()
            if set1 != set2:
                return (False, f"La variable '{name}' n'a pas les mêmes catégories.\n"
                               f"Fichier1={set1}\nFichier2={set2}")

    # 2) Vérifier qu'il n'y a pas de variables supplémentaires dans domain_info2
    dict_info1 = {v["name"]: v for v in domain_info1}
    for var_name in dict_info2:
        if var_name not in dict_info1:
            return (False, f"La variable '{var_name}' est présente dans le second JSON "
                           "mais pas dans le premier.")

    # Si on arrive ici, tout va bien

    return (True, "")


def compare_two_tab_files_as_json(file1, file2):
    """
    1) Lit deux fichiers .tab avec Orange,
    2) Les convertit en JSON,
    3) Compare les deux JSON (structure),
    4) Affiche un message final et renvoie True/False.
    """
    # 1) Récupérer la structure python
    domain_info_1 = parse_tab_file_orange(file1)
    domain_info_2 = parse_tab_file_orange(file2)

    # 2) Convertir en JSON
    json_str_1 = domain_info_to_json(domain_info_1)
    json_str_2 = domain_info_to_json(domain_info_2)

    # 3) Reconvertir le JSON en listes/dicos Python
    data1 = json.loads(json_str_1)
    data2 = json.loads(json_str_2)

    # Comparer
    ok, msg = compare_domain_info(data1, data2)

    if ok:
        print("✔ Les deux fichiers .tab (via JSON) correspondent parfaitement !")
        return True
    else:
        print("✖ Les deux fichiers .tab (via JSON) ne correspondent pas.")
        print("Raison :", msg)
        return False


# --------------------------------------------------------------------
# Exemple d'utilisation
# --------------------------------------------------------------------
# if __name__ == "__main__":
file1 = "C:/toto.tab"
file2 = "C:/totoo.tab"
# domain_info_1 = parse_tab_file_orange(file1)
# json_str_1 = domain_info_to_json(domain_info_1)
# data1 = json.loads(json_str_1)
#
# domain_info_2 = parse_tab_file_orange(file2)
# json_str_2 = domain_info_to_json(domain_info_2)
# data2 = json.loads(json_str_2)
compare_two_tab_files_as_json(file1, file2)


import os
import sys

from AnyQt.QtWidgets import QApplication
from Orange.widgets import widget
from Orange.widgets.utils.signals import Output
from sentence_transformers import SentenceTransformer


if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from orangecontrib.AAIT.utils import SimpleDialogQt, thread_management
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
    from Orange.widgets.orangecontrib.AAIT.utils.MetManagement import GetFromRemote, get_local_store_path
    from Orange.widgets.orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
else:
    from orangecontrib.AAIT.utils import SimpleDialogQt, thread_management
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils.MetManagement import GetFromRemote, get_local_store_path
    from orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file

@apply_modification_from_python_file(filepath_original_widget=__file__)
class OWModelMPNET(widget.OWWidget):
    name = "Model - Embeddings - MPNET"
    description = "Load the embeddings model all-mpnet-base-v2 from the AAIT Store"
    icon = "icons/owmodel_mpnet.svg"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/owmodel_mpnet.svg"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owmodel_mpnet.ui")
    priority = 1061
    want_control_area = False

    class Outputs:
        out_model = Output("Model", SentenceTransformer, auto_summary=False)

    def __init__(self):
        super().__init__()
        # Path management
        self.current_ows = ""
        local_store_path = get_local_store_path()
        model_name = "all-mpnet-base-v2"
        self.model_path = os.path.join(local_store_path, "Models", "NLP", model_name)
        self.model = None

        # Qt Management
        self.setFixedWidth(470)
        self.setFixedHeight(300)
        uic.loadUi(self.gui, self)
        if not os.path.exists(self.model_path):
            if not SimpleDialogQt.BoxYesNo("Model isn't in your computer. Do you want to download it from AAIT store?"):
                return
            try:
                GetFromRemote("Advanced Text Embeddings")
            except:  # TODO ciblage de l'erreur
                SimpleDialogQt.BoxError("Unable to get the Model.")
                return
        # Data Management
        self.progressBarInit()
        self.thread = thread_management.Thread(self.load_sentence_transformer, self.model_path)
        self.thread.finish.connect(self.handle_loading_finish)
        self.thread.start()

    def load_sentence_transformer(self, model_path):
        self.model = SentenceTransformer(model_path)

    def handle_loading_finish(self):
        if self.model is not None:
            self.Outputs.out_model.send(self.model)
        else:
            SimpleDialogQt.BoxError("An Error Occurred when loading model.")
            self.Outputs.out_model.send(None)
        self.progressBarFinished()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = OWModelMPNET()
    my_widget.show()
    app.exec_()



import os
from datetime import datetime
import MetManagement as MetManagement

# Liste des dossiers à ignorer
ignored_dirs = {".git", ".bin", "lfs", "__pycache__", ".svn", ".hg", "node_modules"}

# Fonction pour récupérer les informations des dossiers spécifiques
def get_dir_info(directory, allowed_dirs, source_label):
    """
    Récupère les informations des dossiers uniquement dans les dossiers spécifiques.

    :param directory: Chemin du dossier à analyser.
    :param allowed_dirs: Liste des sous-dossiers autorisés à analyser.
    :param source_label: Label pour indiquer la source (local ou remote).
    :return: Un dictionnaire contenant les chemins relatifs des dossiers avec leur taille totale et leur date de modification.
    """
    dir_info = {}
    print(f"\nAnalyse des dossiers dans '{source_label}': {directory}")
    print(f"{'Dossier':<70} {'Date modification':<25} {'Taille totale (bytes)':<20}")
    print("-" * 120)

    for root, dirs, files in os.walk(directory):
        # Filtrer les dossiers à ignorer
        dirs[:] = [d for d in dirs if d not in ignored_dirs]

        # Vérifier si le chemin contient l'un des sous-dossiers autorisés
        relative_path = os.path.relpath(root, directory).replace("\\", "/")
        if any(relative_path.startswith(d) for d in allowed_dirs) or relative_path in allowed_dirs:
            # Calculer la taille totale des fichiers dans le dossier
            total_size = sum(os.path.getsize(os.path.join(root, file)) for file in files)
            dir_stats = os.stat(root)

            dir_info[relative_path] = {
                'size': total_size,
                'modified': datetime.fromtimestamp(dir_stats.st_mtime).isoformat()
            }

            print(f"{relative_path:<70} {dir_info[relative_path]['modified']:<25} {dir_info[relative_path]['size']:<20}")

    return dir_info

# Fonction pour comparer les dossiers
def compare_directories(local_dir, shared_dir, allowed_dirs):
    """
    Compare les dossiers en se basant uniquement sur les informations des dossiers.

    :param local_dir: Chemin du dossier local.
    :param shared_dir: Chemin du dossier partagé (remote).
    :param allowed_dirs: Liste des sous-dossiers autorisés à comparer.
    :return: Tuple contenant un booléen (True si différences, False sinon) et un tableau de dossiers avec différences.
    """
    # Récupérer les informations des dossiers pour les deux répertoires
    shared_dirs = get_dir_info(shared_dir, allowed_dirs, "remote")
    local_dirs = get_dir_info(local_dir, allowed_dirs, "local")

    deltas = []

    # Comparer les dossiers partagés avec ceux en local
    print("\nComparaison des dossiers entre remote et local :")
    print(f"{'Dossier':<70} {'Date modif remote':<25} {'Taille remote':<20} {'Date modif locale':<25} {'Taille locale':<20}")
    print("-" * 150)

    for dir_path, shared_info in shared_dirs.items():
        if dir_path in local_dirs:
            local_info = local_dirs[dir_path]
            date_diff = shared_info['modified'] != local_info['modified']
            size_diff = shared_info['size'] != local_info['size']

            # Afficher les détails du dossier
            print(f"{dir_path:<70} {shared_info['modified']:<25} {shared_info['size']:<20} {local_info['modified']:<25} {local_info['size']:<20}")

            # Ajouter à la liste des différences si nécessaire
            if date_diff or size_diff:
                deltas.append(dir_path)
        else:
            # Dossier présent dans le remote mais absent en local
            print(f"{dir_path:<70} {shared_info['modified']:<25} {shared_info['size']:<20} {'Absent':<25} {'Absent':<20}")
            deltas.append(dir_path)

    return (len(deltas) > 0, deltas)

# Sous-dossiers spécifiques à analyser
allowed_dirs = ["AddOn", "Models", "Parameters", "Workflows"]

# Chemins des dossiers local et distant
local_dir = MetManagement.get_local_store_path()
shared_dir = MetManagement.get_aait_store_remote_ressources_path()

# Comparaison des dossiers
has_differences, differences = compare_directories(local_dir, shared_dir, allowed_dirs)

# Afficher le résultat final
print("\nRésultat de la comparaison :")
if not has_differences:
    print("Dossier à jour !")
else:
    print("Les dossiers avec des différences sont :", differences)

# Exporter le résultat vers un fichier Excel si des différences existent
if has_differences:
    print("\nCréation d'un DataFrame pour les dossiers avec des différences...")
    df = pd.DataFrame({'Dossier': differences})
    output_file = "comparison_differences_directories.xlsx"
    print(f"Exportation des résultats dans le fichier Excel : {output_file}...")
    df.to_excel(output_file, index=False)
    print(f"Les résultats de la comparaison ont été enregistrés dans {output_file}.")



import subprocess
import json
import time

# # Step 1: Start the workflow
# start_command = [
#     "curl",
#     "-X", "GET",
#     "http://localhost:8000/start-workflow/Acuity_RAG",
#     "-H", "accept: application/json"
# ]
#
# start_result = subprocess.run(start_command, capture_output=True, text=True)
# print("Start Workflow - Status Code:", start_result.returncode)
# print("Start Workflow - Output:", start_result.stdout)
#
# time.sleep(30)


#########################
#########################


# Step 2: Send input data
question = "Qu'est-ce qu'un millésime ?"
workflow_id = "Acuity.ows"
input_id = "chatbotInput1"

payload = {
    "workflow_id": workflow_id,
    "data": [
        {
            "num_input": input_id,
            "values": [
                ["content"],
                ["str"],
                [[question]]
            ]
        }
    ]
}

json_data = json.dumps(payload)
escaped_json = json_data.replace('"', '\\"')  # Escape quotes for shell

input_command = f"""curl --location "http://127.0.0.1:8000/input-workflow" \
--header "Content-Type: application/json" \
--data "{escaped_json}" """

input_result = subprocess.run(input_command, shell=True, capture_output=True, text=True)
print("\nSend Input - STDOUT:\n", input_result.stdout)
print("Send Input - STDERR:\n", input_result.stderr)
print("Send Input - Return Code:", input_result.returncode)





# Step 3: Poke the workflow to get the progress / results
print("\nWaiting for workflow to complete...")
output_command = ["curl", "--location", f"http://127.0.0.1:8000/output-workflow/{workflow_id}"]
timeout = 600
k = 0
while k < timeout:
    output_result = subprocess.run(output_command, capture_output=True)
    stdout_utf8 = output_result.stdout.decode("utf-8")
    parsed_response = json.loads(stdout_utf8)
    print(parsed_response)
    if parsed_response["_result"] is None:
        print("Progress:", parsed_response["_statut"])
    else:
        break
    time.sleep(1)
    k += 1
print("\nRaw Output:\n", stdout_utf8)





# Step 4: Parse and get the results
try:
    parsed_response = json.loads(stdout_utf8)
    print("\nParsed Response:\n", parsed_response)
    question = parsed_response["_result"][0]["content"]
    answer = parsed_response["_result"][0]["Answer"]
    print("\nQuestion:", question)
    print("\nAnswer:", answer)
except json.JSONDecodeError as e:
    print("Failed to parse JSON:", e)
except (KeyError, IndexError) as e:
    print("Unexpected response structure:", e)



# .bat pour lancer server
# .py pour lancer workflow sans GUI
# faire fichier dans Parameters
# rajouter while pour voir le progress


import os
import gc
import sys
import json
from Orange.data import Table, Domain, StringVariable
from AnyQt.QtWidgets import QApplication
from AnyQt.QtCore import QEventLoop

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\","/"):
     from Orange.widgets.orangecontrib.AAIT.widgets.OWLLM4ALL import OWLLM4ALL
     from Orange.widgets.orangecontrib.AAIT.widgets.OWModel_Solar import OWModelSolar
     from Orange.widgets.orangecontrib.AAIT.audit_widget.SignalReceiver import SignalReceiver
     from Orange.widgets.orangecontrib.AAIT.audit_widget.widgets_model import check_model_path
else:
     from orangecontrib.AAIT.widgets.OWLLM4ALL import OWLLM4ALL
     from orangecontrib.AAIT.widgets.OWModel_Solar import OWModelSolar
     from orangecontrib.AAIT.audit_widget.SignalReceiver import SignalReceiver
     from orangecontrib.AAIT.audit_widget.widgets_model import check_model_path


def check_llm4all(llm4all, model, receiver):
     json_data = '[{"prompt": "Quand marseille a-t-il gagné la coupe d Europe? "}]'
     data = json.loads(json_data)
     text_var = StringVariable("prompt")
     domain = Domain([], metas=[text_var])
     rows = [[item["prompt"]] for item in data]
     table = Table.from_list(domain, rows)

     llm4all.set_data(table)
     llm4all.set_model(model)
     loop = QEventLoop()

     def on_finish():
         loop.quit()
     llm4all.thread.finish.connect(on_finish)
     loop.exec_()
     llm4all.Outputs.data.send = receiver.receive_data
     llm4all.Outputs.data.send(llm4all.result)
     data = receiver.received_data
     if data[0]["Answer"].value != None and len(data[0]["Answer"].value) > 0:
          return 0
     else:
          return 1


def check_solar(solar, receiver):
     solar.Outputs.model.send = receiver.receive_data
     solar.Outputs.model.send(solar.model_name)
     model = receiver.received_data
     if model == None:
         print("Erreur au chargement du model solar")
         exit(0)
     return model

def check_widget_llm4all():
     app = QApplication(sys.argv)
     llm4all = OWLLM4ALL()
     solar = OWModelSolar()
     receiver = SignalReceiver()
     model = check_model_path(solar, receiver, "solar")
     if "solar" not in model:
           print("Erreur au chargement du model solar")
           return 1
     if check_llm4all(llm4all, model, receiver) != 0:
           print("Erreur à la génération de réponse de 4all")
           return 1
     return 0


if __name__ == "__main__":
     check_widget_llm4all()






